@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploaderConfig.h 2756 2007-12-11 10:57:59Z svn $
+ * $Id: UploaderConfig.h 2768 2007-12-16 13:03:37Z svn $
  *****************************************************************************/
 
 #ifndef UPLOADER_CONFIG_H
@@ -69,6 +69,7 @@ public:
         __TMPL_COUNT__
     };
 
+    static const item_t PATH;
     static const item_t BASE_URL;
     static const item_t DATA_DIRECTORY;
     static const item_t FILE_DIRECTORY;
@@ -137,6 +138,7 @@ public:
     const char *to_string(apr_pool_t *pool);
 
     // 面倒なので public
+    const char *path;
     const char *base_url;
     const char *data_dir_path;
     const char *file_dir_path;
