@@ -1,5 +1,5 @@
 /******************************************************************************
- * Copyright (C) 2006 Tetsuya Kimata <kimata@acapulco.dyndns.org>
+ * Copyright (C) 2007 Tetsuya Kimata <kimata@acapulco.dyndns.org>
  *
  * All rights reserved.
  *
@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: ReadWriteLocker.cpp 2732 2007-12-09 06:31:18Z svn $
+ * $Id: ReadWriteLocker.cpp 2758 2007-12-11 13:45:39Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -30,7 +30,7 @@
 #include "ReadWriteLocker.h"
 #include "SourceInfo.h"
 
-SOURCE_INFO_ADD("$Id: ReadWriteLocker.cpp 2732 2007-12-09 06:31:18Z svn $");
+SOURCE_INFO_ADD("$Id: ReadWriteLocker.cpp 2758 2007-12-11 13:45:39Z svn $");
 
 
 /******************************************************************************
@@ -79,6 +79,8 @@ const apr_size_t READ_COUNT             = 30;
 const apr_size_t WRITE_COUNT            = 3;
 const apr_size_t DIE_COUNT              = 3;
 const apr_size_t REPEAT_COUNT           = 100;
+const apr_size_t RETRY_COUNT            = 1000;
+const apr_size_t TIME_LIMIT_SEC         = 60;
 
 typedef struct {
     apr_atomic_t lock;
@@ -102,20 +104,17 @@ void read_lock(void *data)
 
     handle = reinterpret_cast<handle_t *>(data);
 
-    ReadLocker read_lock(&(handle->lock));
-
     apr_sleep(rand() & 0xFF);
     ReadWriteLocker::yield();
+
+    ReadLocker read_lock(&(handle->lock));
+
     apr_atomic_inc(&(handle->read_count));
 
     if (read_lock.is_process_timeout()) {
         apr_atomic_inc(&(handle->timeout_count));
     }
 
-    if (ReadWriteLocker::get_read_count(handle->lock) <
-        apr_atomic_read(&(handle->read_count))) {
-        THROW(MESSAGE_BUG_FOUND);
-    }
     if (ReadWriteLocker::get_read_count(handle->lock) ==
         ReadWriteLocker::WRITE_LOCKED) {
         THROW(MESSAGE_BUG_FOUND);
@@ -123,6 +122,7 @@ void read_lock(void *data)
 
     apr_sleep(rand() & 0xFF);
     ReadWriteLocker::yield();
+
     apr_atomic_dec(&(handle->read_count));
 }
 
@@ -146,11 +146,12 @@ void write_lock(void *data)
             ReadWriteLocker::WRITE_LOCKED) {
             THROW(MESSAGE_BUG_FOUND);
         }
+
         if (apr_atomic_read(&(handle->read_count)) != 0) {
             THROW(MESSAGE_BUG_FOUND);
         }
 
-        apr_sleep(rand() & 0xFF);
+        apr_sleep((rand() & 0xFF) << 4);
         ReadWriteLocker::yield();
     }
 }
@@ -240,16 +241,21 @@ void run_read(apr_pool_t *pool)
             is_ok = false;
         }
     }
-    if (ReadWriteLocker::get_read_count(handle.lock) != 0) {
-        is_ok = false;
-    }
-
     if (!is_ok) {
         THROW(MESSAGE_BUG_FOUND);
     }
 
+    if (ReadWriteLocker::get_read_count(handle.lock) != 0) {
+        THROW(MESSAGE_BUG_FOUND);
+    }
+
     end_time = get_time_sec();
 
+    if ((end_time - start_time) > TIME_LIMIT_SEC) {
+        // 時間がかかりすぎた場合もダメ
+        THROW(MESSAGE_BUG_FOUND);
+    }
+
     show_item("read", (end_time - start_time)*1000, " msec");
 }
 
@@ -260,16 +266,92 @@ void run_write(apr_pool_t *pool)
     work_t write_work;
     apr_thread_t *read_workers[READ_COUNT];
     apr_thread_t *write_workers[WRITE_COUNT];
-    apr_thread_t *read_die_workers[DIE_COUNT];
-    apr_thread_t *write_die_workers[DIE_COUNT];
     apr_status_t status;
-    bool is_ok;
+    bool is_read_ok;
+    bool is_write_ok;
     volatile double start_time;
     volatile double end_time;
 
     show_test_name("write");
 
-    is_ok = true;
+    is_read_ok = true;
+    is_write_ok = true;
+
+    handle.lock = 0;
+    handle.read_count = 0;
+    handle.timeout_count = 0;
+
+    read_work.handle = &handle;
+    read_work.func = read_lock;
+    write_work.handle = &handle;
+    write_work.func = write_lock;
+
+    start_time = get_time_sec();
+
+    for (apr_size_t i = 0; i < READ_COUNT; i++) {
+        apr_thread_create(read_workers + i, NULL, worker, &read_work, pool);
+    }
+    for (apr_size_t i = 0; i < WRITE_COUNT; i++) {
+        apr_thread_create(write_workers + i, NULL, worker,
+                          &write_work, pool);
+    }
+
+    for (apr_size_t i = 0; i < READ_COUNT; i++) {
+        apr_thread_join(&status, read_workers[i]);
+        if (status != APR_SUCCESS) {
+            is_read_ok = false;
+        }
+    }
+    for (apr_size_t i = 0; i < WRITE_COUNT; i++) {
+        apr_thread_join(&status, write_workers[i]);
+        if (status != APR_SUCCESS) {
+            is_write_ok = false;
+        }
+    }
+    if (!is_read_ok) {
+        THROW(MESSAGE_BUG_FOUND);
+    } else if (!is_write_ok) {
+        THROW(MESSAGE_BUG_FOUND);
+    }
+
+    if (ReadWriteLocker::get_read_count(handle.lock) != 0) {
+        THROW(MESSAGE_BUG_FOUND);
+    }
+    if (apr_atomic_read(&(handle.timeout_count)) != 0) {
+        THROW(MESSAGE_BUG_FOUND);
+    }
+
+    end_time = get_time_sec();
+
+    if ((end_time - start_time) > TIME_LIMIT_SEC) {
+        // 時間がかかりすぎた場合もダメ
+        THROW(MESSAGE_BUG_FOUND);
+    }
+
+    show_item("write", (end_time - start_time)*1000, " msec");
+}
+
+void run_die(apr_pool_t *pool)
+{
+    handle_t handle;
+    work_t read_work;
+    work_t write_work;
+    apr_thread_t *read_workers[READ_COUNT];
+    apr_thread_t *write_workers[WRITE_COUNT];
+    apr_thread_t *read_die_workers[DIE_COUNT];
+    apr_thread_t *write_die_workers[DIE_COUNT];
+    apr_status_t status;
+    bool is_read_ok;
+    bool is_write_ok;
+    bool is_die_ok;
+    volatile double start_time;
+    volatile double end_time;
+
+    show_test_name("die");
+
+    is_read_ok = true;
+    is_write_ok = true;
+    is_die_ok = true;
 
     handle.lock = 0;
     handle.read_count = 0;
@@ -301,30 +383,36 @@ void run_write(apr_pool_t *pool)
     for (apr_size_t i = 0; i < READ_COUNT; i++) {
         apr_thread_join(&status, read_workers[i]);
         if (status != APR_SUCCESS) {
-            is_ok = false;
+            is_read_ok = false;
         }
     }
     for (apr_size_t i = 0; i < WRITE_COUNT; i++) {
         apr_thread_join(&status, write_workers[i]);
         if (status != APR_SUCCESS) {
-            is_ok = false;
+            is_write_ok = false;
         }
     }
     for (apr_size_t i = 0; i < DIE_COUNT; i++) {
         apr_thread_join(&status, read_die_workers[i]);
         if (status != APR_SUCCESS) {
-            is_ok = false;
+            is_die_ok = false;
         }
     }
-    for (apr_size_t i = 0; i < DIE_COUNT; i++) {
-        apr_thread_join(&status, write_die_workers[i]);
-        if (status != APR_SUCCESS) {
-            is_ok = false;
-        }
+    if (!is_read_ok) {
+        THROW(MESSAGE_BUG_FOUND);
+    } else if (!is_write_ok) {
+        THROW(MESSAGE_BUG_FOUND);
+    } else if (!is_die_ok) {
+        THROW(MESSAGE_BUG_FOUND);
     }
 
-    if (ReadWriteLocker::get_read_count(handle.lock) != 0) {
-        is_ok = false;
+    if ((ReadWriteLocker::get_read_count(handle.lock) > DIE_COUNT) &&
+        (ReadWriteLocker::get_read_count(handle.lock) !=
+         ReadWriteLocker::WRITE_LOCKED)) {
+        // MEMO: 最後にロックを保持したまま終了した場合，
+        // get_read_count の返り値は DIE_COUNT 以下や WRITE_LOCKED になっ
+        // たりする可能性がある．
+        THROW(MESSAGE_BUG_FOUND);
     }
     if (apr_atomic_read(&(handle.timeout_count)) < (DIE_COUNT + 1) ||
         apr_atomic_read(&(handle.timeout_count)) > (DIE_COUNT * 2)) {
@@ -332,16 +420,17 @@ void run_write(apr_pool_t *pool)
         // したまま終了させているが，Read に関してはネストしてロックで
         // きるので，タイムアウト回数の期待値としては DIE_COUNT + 1 以
         // 上 DIE_COUNT * 2 以下となる．
-        is_ok = false;
-    }
-
-    if (!is_ok) {
         THROW(MESSAGE_BUG_FOUND);
     }
 
     end_time = get_time_sec();
 
-    show_item("write", (end_time - start_time)*1000, " msec");
+    if ((end_time - start_time) > TIME_LIMIT_SEC) {
+        // 時間がかかりすぎた場合もダメ
+        THROW(MESSAGE_BUG_FOUND);
+    }
+
+    show_item("die", (end_time - start_time)*1000, " msec");
 }
 
 void run_all(apr_pool_t *pool, int argc, const char * const *argv)
@@ -357,9 +446,11 @@ void run_all(apr_pool_t *pool, int argc, const char * const *argv)
 #if APR_HAS_THREADS
     run_read(pool);
     run_write(pool);
+    run_die(pool);
 #else
     show_item("read", "skipped");
     show_item("write", "skipped");
+    show_item("die", "skipped");
 #endif
 }
 
