@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItem.cpp 2756 2007-12-11 10:57:59Z svn $
+ * $Id: UploadItem.cpp 2762 2007-12-16 12:03:46Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -40,7 +40,7 @@
 #include "Auxiliary.h"
 #include "SourceInfo.h"
 
-SOURCE_INFO_ADD("$Id: UploadItem.cpp 2756 2007-12-11 10:57:59Z svn $");
+SOURCE_INFO_ADD("$Id: UploadItem.cpp 2762 2007-12-16 12:03:46Z svn $");
 
 #define AS_BCOMMENT(pointer)        reinterpret_cast<bbs_comment_t *>(pointer)
 #define UINT16(pointer)             static_cast<apr_uint16_t>(pointer)
@@ -88,6 +88,7 @@ UploadItem *UploadItem::get_instance(void *memory, apr_size_t item_id,
     uitem->header_.id           = item_id;
     uitem->header_.index        = item_index;
     uitem->header_.mtime        = mtime;
+    uitem->header_.atime        = mtime;
     uitem->header_.file_size    = file_size;
 
 #ifdef USE_GZIP
@@ -156,6 +157,24 @@ int UploadItem::cmp_file_mime(const UploadItem *a, const UploadItem *b)
 {
     return strcmp(b->get_file_mime(), a->get_file_mime());
 }
+
+int UploadItem::cmp_file_atime(const UploadItem *a, const UploadItem *b)
+{
+    apr_time_t a_atime;
+    apr_time_t b_atime;
+
+    a_atime = a->get_atime();
+    b_atime = b->get_atime();
+
+    if (b_atime > a_atime) {
+        return -1;
+    } else if (b_atime == a_atime) {
+        return 0;
+    } else {
+        return 1;
+    }
+}
+
 int UploadItem::cmp_comment(const UploadItem *a, const UploadItem *b)
 {
     return strcmp(b->get_comment(), a->get_comment());
