@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: uploader_handler.cpp 2756 2007-12-11 10:57:59Z svn $
+ * $Id: uploader_handler.cpp 2772 2008-01-24 16:25:23Z svn $
  *****************************************************************************/
 
 #ifndef TEMPLATE_INSTANTIATION
@@ -57,7 +57,7 @@
 #include "SourceInfo.h"
 
 #ifndef TEMPLATE_INSTANTIATION
-SOURCE_INFO_ADD("$Id: uploader_handler.cpp 2756 2007-12-11 10:57:59Z svn $");
+SOURCE_INFO_ADD("$Id: uploader_handler.cpp 2772 2008-01-24 16:25:23Z svn $");
 #endif
 
 #define STR_CMP_LITERAL(arg, literal)                            \
@@ -933,9 +933,10 @@ static int download_impl(typename Response::Handle *r, UploaderConfig *config,
 
 template<class Response>
 static int download_prep(typename Response::Handle *r, UploaderConfig *config,
-                         UploadItem *uitem)
+                         UploadItem *uitem, bool is_admin_mode=false)
 {
     const char *file_name;
+    const char *command;
     int status;
 
     if (Response::is_head_method(r)) {
@@ -959,15 +960,23 @@ static int download_prep(typename Response::Handle *r, UploaderConfig *config,
          CharCodeConverter::DEFAULT_CODE, "utf-8");
 #endif
 
+    if (is_admin_mode) {
+        command = apr_pstrcat(r->pool, ADMIN_COMMAND, ARG_SEPARATE_STR,
+                              DOWNLOAD_COMMAND, NULL);
+    } else {
+        command = DOWNLOAD_COMMAND;
+    }
+
     typename Response::Writer o(r);
 
     return redirect<Response>
         (r, o,
-         apr_pstrcat(r->pool, config->base_url, ARG_SEPARATE_STR,
-                     DOWNLOAD_COMMAND, ARG_SEPARATE_STR,
-                     apr_psprintf(r->pool, "%" APR_SIZE_T_FMT, uitem->get_id()),
-                     ARG_SEPARATE_STR, "attach", ARG_SEPARATE_STR,
-                     rfc2396_encode(r->pool, file_name), NULL));
+         apr_pstrcat
+         (r->pool, config->base_url, ARG_SEPARATE_STR,
+          command, ARG_SEPARATE_STR,
+          apr_psprintf(r->pool, "%" APR_SIZE_T_FMT, uitem->get_id()),
+          ARG_SEPARATE_STR, "attach", ARG_SEPARATE_STR,
+          rfc2396_encode(r->pool, file_name), NULL));
 }
 
 template<class Response>
@@ -1059,6 +1068,7 @@ static int download(typename Response::Handle *r, UploaderConfig *config,
         }
 #endif
 #endif
+        config->get_item_manager()->update_atime(item_id);
 
         if (is_view_mode) {
             // 表示
@@ -1067,7 +1077,7 @@ static int download(typename Response::Handle *r, UploaderConfig *config,
         } else {
             if (strlen(get_word(r->pool, &arg, ARG_SEPARATE_STR[0])) == 0) {
                 // ファイル名を追加した URL に飛ばす
-                return download_prep<Response>(r, config, uitem);
+                return download_prep<Response>(r, config, uitem, is_admin_mode);
             } else {
                 // ダウンロード
                 return download_impl<Response>(r, config, uitem, item_file,
